import re
import pytesseract
from PIL import Image


def extract_names(input):
    regex_swd = r"dual\sOwner\)?\s*:?\s*([A-Z.]+\s[A-Z.]+\s[A-Z.]+)"
    match = re.search(regex_swd, input, re.IGNORECASE)
    swd = match.group(1) if match else ""

    regex_name = r"NAME\s*:?\s*([A-Z]+\s[A-Z]+)"
    match_name = re.search(regex_name, input, re.IGNORECASE)
    name = match_name.group(1) if match_name else ""

    if swd == "":
        regex_swd = r"NAME\s*:?\s*([A-Z]+)\s"
        match_swd = re.findall(regex_swd, input, re.IGNORECASE)
        swd = match_swd[1] if len(match_swd) > 1 else ""

    if swd == "":
        regex_swd = r"OF\s*:?\s*[S/O01]*\s*([A-Z]+\s[A-Z]+)"
        match = re.search(regex_swd, input, re.IGNORECASE)
        swd = match.group(1) if match else ""

    return name, swd


def extract_reg_number(input):
    regex = r"(?=.*\d)[A-Z0-9]{10}"
    match = re.search(regex, input)
    reg_number = match.group(0) if match else ""

    return reg_number


def extract_chasis(input):
    regex = r"[A-Z0-9]{17,18}"
    match = re.search(regex, input)
    chasis = match.group(0) if match else ""

    return chasis


def extract_fuel_type(input):
    regex = r"Fuel(?:\s+Type)?\s*[\s:\.]\s*([A-Z/]+)\s"
    match = re.search(regex, input, re.IGNORECASE)
    fuel_type = match.group(1) if match else ""
    return fuel_type


def extract_vehicle_class(input):
    regex = r"(?:Veh.c.e\sClass|Veh\sCl)\s*[\s:]\s*([A-Z0-9/()-]+)\s([A-Z0-9/()-]+)\s"
    match = re.search(regex, input, re.IGNORECASE)
    vehicle_class = match.group(1) if match else ""
    return vehicle_class


def extract_manufacturer(input):
    regex = r"MFR\s*:\s*([A-Z\s]+)\n"
    match = re.search(regex, input, re.IGNORECASE)
    manufacturer = match.group(1) if match else ""
    return manufacturer


def extract_tax_info(input):
    regex = r"Tax\sUp\s{0,1}to\s*:\s*([A-Z]+)\s"
    match = re.search(regex, input, re.IGNORECASE)
    tax_up_to = match.group(1) if match else ""
    return tax_up_to


def extract_model(input):
    regex = r"Mode.\s*[\s:]\s*([A-Z0-9/+()-.]+(?:\s+[^\w\n]*[A-Z0-9/+()-.]+){0,3})\s"
    match = re.search(regex, input, re.IGNORECASE)
    model = match.group(1) if match else ""
    return model


def extract_all_dates(input_text):
    regex = r"\b(\d{1,2}[/\-.](?:\d{2}|\d{4}|\w{3})[/\-.]\d{2,4})\b"
    dates = re.findall(regex, input_text)
    sorted_dates = sorted(
        dates, key=lambda date: int(date.split("/")[-1].split("-")[-1])
    )

    return sorted_dates


def extract_address(input):
    regex = r"Address:?\s*((?:.|\n)*?\d{6})"
    match = re.search(regex, input, re.IGNORECASE)
    address = match.group(1) if match else ""

    return address


def extract_vehicle_registration_details(image_path):
    image = Image.open(image_path)
    extracted_text = pytesseract.image_to_string(image)

    names = extract_names(extracted_text)
    name, swd = names[0], names[1]

    dates = extract_all_dates(extracted_text)
    expiry_date = dates[-1] if len(dates) > 0 else ""
    registration_date = dates[0] if len(dates) > 1 else ""

    chasis_number = extract_chasis(extracted_text)
    reg_number = extract_reg_number(extracted_text)

    address = extract_address(extracted_text)

    fuel_type = extract_fuel_type(extracted_text)
    vehicle_class = extract_vehicle_class(extracted_text)
    model = extract_model(extracted_text)

    manufacturer = extract_manufacturer(extracted_text)
    tax_up_to = extract_tax_info(extracted_text)

    return {
        "Registration Number": reg_number,
        "Chasis Number": chasis_number,
        "Full Name": name,
        "S/W/D of": swd,
        "Address": address,
        "Fuel Type": fuel_type,
        "Vehicle Class": vehicle_class,
        "Vehicle Model": model,
        "Manufacturer": manufacturer,
        "Registration Date": registration_date,
        "Expiry Date": expiry_date,
        "Tax Upto": tax_up_to,
    }


def vehicle_registration(image_path):
    return extract_vehicle_registration_details(image_path)



import re
import pytesseract
from PIL import Image
from datetime import datetime


def preprocess_for_bold_text(image):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (1, 1))
    opening = cv2.morphologyEx(gray, cv2.MORPH_OPEN, kernel)
    contrast = cv2.addWeighted(opening, 2, opening, -0.5, 0)

    _, binary = cv2.threshold(contrast, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
    sharpened = cv2.filter2D(
        binary, -1, np.array([[0, -1, 0], [-1, 5, -1], [0, -1, 0]])
    )

    return sharpened


def extract_name(input):
    regex = r"Name:\s*(.*?)(?:\.\s|(?=\n))"
    match = re.search(regex, input)
    name = match.group(1).strip() if match else ""

    if name == "":
        regex = r"(?:Mr\sMrs\s*[:\s]?\s*)(.*?)(?:\bConsumer\b|['/]|$)"
        match = re.search(regex, input, re.IGNORECASE)
        name = match.group(1).strip() if match else ""

    return name


def extract_bill_amount(input):
    regex = r"Bill Amount \(Rs\.\)\s*:? (\d+)"
    match = re.search(regex, input, re.IGNORECASE)
    bill_amount = match.group(1).strip() if match else ""
    return bill_amount


def extract_meter_number(input):
    regex = r"Meter No\.\s*:\s*(\d+|NA)"
    match = re.search(regex, input, re.IGNORECASE)
    meter_number = match.group(1).strip() if match else ""
    return meter_number


def extract_all_dates(input):
    regex = r"\b(\d{1,2}-[A-Z]{3}-\d{4})\b"
    dates = re.findall(regex, input)
    formatted_dates = []
    for date in dates:
        try:
            formatted_date = datetime.strptime(date, "%d-%b-%Y")
            formatted_dates.append(formatted_date)
        except ValueError:
            continue
    sorted_dates = sorted(formatted_dates)
    sorted_dates_str = [date.strftime("%d-%m-%Y") for date in sorted_dates]
    return sorted_dates_str


def extract_phone(input):
    regex = r"Mobile No\.\s*:\s*(\d+)"
    match = re.search(regex, input)
    phone = match.group(1).strip() if match else ""
    return phone


def extract_address(input):
    regex = r"Address\s*:\s*(.*?)(?=\s*[A-Z][a-zA-Z\s]*:|$)"
    match = re.search(regex, input, re.DOTALL | re.IGNORECASE)
    address = match.group(1).strip() if match else ""
    return address


def extract_mr_code(input):
    regex = r"Zone/MR\s*Code:\s*([A-Z0-9/]+\s*[A-Z0-9/]*)"
    match = re.search(regex, input, re.IGNORECASE)
    mr_code = match.group(1).strip() if match else ""
    return mr_code


def extract_area_code(input):
    regex = r"Area Code\s*:\s*([\w/-]+)"
    match = re.search(regex, input, re.IGNORECASE)
    area_code = match.group(1).strip() if match else ""
    return area_code


def extract_bill_number(input):
    regex = r"Bill No\.\s*(?::\s*)?(\d+)"
    match = re.search(regex, input)
    bill_number = match.group(1).strip() if match else ""
    return bill_number


def extract_govt_body(input):
    regex = r"Delhi Jal Board"
    match = re.search(regex, input, re.IGNORECASE)
    govt_body = match.group(0).strip() if match else "Unknown"
    return govt_body


def extract_bill_date(input):
    regex = r"Bill Date\s*:? (\d{2}-[A-Z]{3}-\d{4})"
    match = re.search(regex, input, re.IGNORECASE)
    bill_date = match.group(1).strip() if match else ""
    return bill_date


def extract_bill_due_date(input):
    regex = r"Bill Due Date\s*:? (\d{2}-[A-Z]{3}-\d{4})"
    match = re.search(regex, input, re.IGNORECASE)
    bill_due_date = match.group(1).strip() if match else ""
    return bill_due_date


def extract_water_bill_details(image_path):
    image = Image.open(image_path)
    extracted_text = pytesseract.image_to_string(image)

    name = extract_name(extracted_text)
    dates = extract_all_dates(extracted_text)
    bill_date = extract_bill_date(extracted_text)
    due_date = extract_bill_due_date(extracted_text)
    address = extract_address(extracted_text)
    phone = extract_phone(extracted_text)
    bill_amount = extract_bill_amount(extracted_text)
    mr_code = extract_mr_code(extracted_text)
    area_code = extract_area_code(extracted_text)
    meter_number = extract_meter_number(extracted_text)
    bill_number = extract_bill_number(extracted_text)
    govt_body = extract_govt_body(extracted_text)

    return {
        "Name": name,
        "Phone Number": phone,
        "Bill Amount": bill_amount,
        "Bill Date": bill_date,
        "Due Date": due_date,
        "Address": address,
        "Zone/MR Code": mr_code,
        "Area Code": area_code,
        "Meter Number": meter_number,
        "Bill Number": bill_number,
        "Source/Govt Body Name": govt_body,
    }


def water_bill(image_path):
    return extract_water_bill_details(image_path)



# author:    Kunal Kumar Kushwaha
# website:   http://www.essentia.dev

# import the required packages
from openbharatocr.ocr.api import (
    pan,
    front_aadhaar,
    back_aadhaar,
    driving_licence,
    passport,
    voter_id_front,
    voter_id_back,
    vehicle_registration,
    water_bill,
)


__all__ = [
    "pan",
    "front_aadhaar",
    "back_aadhaar",
    "driving_licence",
    "passport",
    "voter_id_front",
    "voter_id_back",
    "vehicle_registration",
    "water_bill",
]



from openbharatocr.ocr.pan import (
    clean_input,
    extract_all_names,
    extract_pan,
    extract_dob,
    extract_pan_details,
)
import pytest
from unittest import mock, TestCase


class Test_clean_input(TestCase):
    def test_clean_input_with_newlines(self):
        match = ["Amit\nKumar", "Sunita\nKumar\nAnil"]
        expected_output = ["Amit", "Kumar", "Sunita", "Kumar", "Anil"]
        assert clean_input(match) == expected_output

    def test_clean_input_without_newlines(self):
        match = ["AMIT KUMAR", "SUNITA KUMAR"]
        expected_output = ["AMIT KUMAR", "SUNITA KUMAR"]
        assert clean_input(match) == expected_output

    def test_clean_input_with_empty_string(self):
        match = [""]
        expected_output = [""]
        assert clean_input(match) == expected_output

    def test_clean_input_with_none(self):
        with pytest.raises(TypeError):
            clean_input(None)


class Test_extract_all_names(TestCase):
    def test_extract_all_names(self):
        input_string = """
        This is a sample input string.
        AMIT KUMAR
        SUNITA KUMAR
        INCOME TAX DEPARTMENT INDIA
        ANIL JOSHI
        GOVT OF INDIA
        """
        expected_output = ["AMIT KUMAR", "SUNITA KUMAR", "ANIL JOSHI"]
        assert extract_all_names(input_string) == expected_output

    def test_extract_all_names_individual(slef):
        assert extract_all_names("") == []

        input_string = "This is a string without any names."
        assert extract_all_names(input_string) == []

        input_string = "INDIA GOVT TAX DEPARTMENT"
        assert extract_all_names(input_string) == []

        input_string = "ABC\nXYZ"
        assert extract_all_names(input_string) == []

    def test_extract_all_names_with_empty_input(self):
        input_text = ""
        expected_output = []
        assert extract_all_names(input_text) == expected_output

    def test_extract_all_names_with_invalid_input(self):
        with pytest.raises(TypeError):
            extract_all_names(None)


class Test_extract_pan(TestCase):
    def test_extract_pan_valid_input(self):
        input_text = "Amit Kumar's PAN number is ABCDE1234F"
        expected_output = "ABCDE1234F"
        assert extract_pan(input_text) == expected_output

    def test_extract_pan_no_match(self):
        input_text = "This is a random text without a PAN number"
        expected_output = ""
        assert extract_pan(input_text) == expected_output

    def test_extract_pan_invalid_format(self):
        input_text = "This is an invalid PAN number: ABCDE12345"
        assert extract_pan(input_text) == ""

    def test_extract_pan_invalid_input(self):
        with pytest.raises(TypeError):
            extract_pan(None)
        with pytest.raises(TypeError):
            extract_pan(123)


class Test_extract_dob(TestCase):
    def test_extract_dob_valid_formats(self):
        inputs = [
            "Amit Kumar's date of birth is 01/01/1990",
            "15-12-1985 is sunita kumar's DOB",
            "Rajiv's birth date is 30.06.78",
        ]
        expected_outputs = ["01/01/1990", "15-12-1985", "30.06.78"]
        for input_str, expected_output in zip(inputs, expected_outputs):
            assert extract_dob(input_str) == expected_output

    def test_extract_dob_invalid_formats(self):
        assert extract_dob("Amit Kumar's DOB is 1990/01/01") == ""
        assert extract_dob("DOB: 01/Jan/1990") == ""
        assert extract_dob("My DOB is 01:01:1990") == ""

        assert extract_dob("DOB: 15 03 1985") == ""

    def test_extract_dob_invalid_input(self):
        with pytest.raises(TypeError):
            extract_dob(None)


class Test_extract_pan_details(TestCase):
    def test_image_without_pan_details(self):
        # Mock the image reading and text extraction
        with mock.patch("PIL.Image.open") as mock_open, mock.patch(
            "pytesseract.image_to_string"
        ) as mock_image_to_string, mock.patch("imghdr.what") as mock_what:
            mock_image = mock.MagicMock()
            mock_open.return_value = mock_image
            mock_image_to_string.return_value = "This is a test without PAN details."
            mock_what.return_value = "jpeg"

            result = extract_pan_details("dummy_img.jpg")

            assert result["Full Name"] == ""
            assert result["Parent's Name"] == ""
            assert result["Date of Birth"] == ""
            assert result["PAN Number"] == ""

            mock_open.assert_called_once_with("dummy_img.jpg")
            mock_image_to_string.assert_called_once_with(mock_image)
            mock_what.assert_called_once_with("dummy_img.jpg")

            with mock.patch(
                "openbharatocr.ocr.pan.extract_all_names"
            ) as mock_extract_all_names, mock.patch(
                "openbharatocr.ocr.pan.extract_dob"
            ) as mock_extract_dob, mock.patch(
                "openbharatocr.ocr.pan.extract_pan"
            ) as mock_extract_pan:
                mock_extract_all_names.return_value = []
                mock_extract_dob.return_value = ""
                mock_extract_pan.return_value = ""

                result = extract_pan_details("dummy_img.jpg")

                assert result["Full Name"] == ""
                assert result["Parent's Name"] == ""
                assert result["Date of Birth"] == ""
                assert result["PAN Number"] == ""

                mock_extract_all_names.assert_called_once_with(
                    "This is a test without PAN details."
                )
                mock_extract_dob.assert_called_once_with(
                    "This is a test without PAN details."
                )
                mock_extract_pan.assert_called_once_with(
                    "This is a test without PAN details."
                )



import re
import cv2
import pytesseract
from PIL import Image
import numpy as np
import os
import tempfile
import uuid

# Download the models from links and set in the environment
YOLO_CFG = os.environ.get(
    "YOLO_CFG", "yolov3_custom.cfg"
)  # https://drive.google.com/file/d/1SEst2lVoFDOgUVLZ5kje9GTb2tHRA8U-/view?usp=sharing
YOLO_WEIGHT = os.environ.get(
    "YOLO_WEIGHT", "yolov3_custom_6000.weights"
)  # https://drive.google.com/file/d/1cGGstycfogmO6O7ToB2DAEXOgTWVgINh/view?usp=drive_link


def preprocess_for_bold_text(image):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (1, 1))
    opening = cv2.morphologyEx(gray, cv2.MORPH_OPEN, kernel)
    contrast = cv2.addWeighted(opening, 2, opening, -0.5, 0)

    _, binary = cv2.threshold(contrast, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
    sharpened = cv2.filter2D(
        binary, -1, np.array([[0, -1, 0], [-1, 5, -1], [0, -1, 0]])
    )

    return sharpened


def extract_voter_details_yolo(image_path):
    image = Image.open(image_path)
    net = cv2.dnn.readNetFromDarknet(YOLO_CFG, YOLO_WEIGHT)
    classes = ["elector", "relation", "voterid"]

    rgb = image.convert("RGB")
    with tempfile.TemporaryDirectory() as tempdir:
        tempfile_path = f"{tempdir}/{str(uuid.uuid4())}.jpg"
        rgb.save(tempfile_path)

        img = cv2.imread(tempfile_path)
        if img is None:
            print("Error: Unable to read the input image.")
            exit()

        height, width, _ = img.shape
        blob = cv2.dnn.blobFromImage(
            img, 1 / 255, (416, 416), (0, 0, 0), swapRB=True, crop=False
        )

        net.setInput(blob)
        output_layers_name = net.getUnconnectedOutLayersNames()
        layerOutputs = net.forward(output_layers_name)

        boxes = []
        confidences = []
        class_ids = []
        detected_texts = {}

        for output in layerOutputs:
            for detection in output:
                scores = detection[5:]
                class_id = np.argmax(scores)
                confidence = scores[class_id]
                if confidence > 0.5:
                    center_x = int(detection[0] * width)
                    center_y = int(detection[1] * height)
                    w = int(detection[2] * width)
                    h = int(detection[3] * height)

                    x = int(center_x - w / 2)
                    y = int(center_y - h / 2)

                    boxes.append([x, y, w, h])
                    confidences.append(float(confidence))
                    class_ids.append(class_id)

        indexes = cv2.dnn.NMSBoxes(boxes, confidences, 0.5, 0.4)

        for i in range(len(boxes)):
            if i in indexes:
                x, y, w, h = boxes[i]

                x = max(0, x)
                y = max(0, y)
                w = min(width - x, w)
                h = min(height - y, h)

                label = str(classes[class_ids[i]])
                crop_img = img[y : y + h, x : x + w]
                if crop_img.size == 0:
                    continue
                text = pytesseract.image_to_string(crop_img, config="--psm 6")
                detected_texts[label] = text.strip()

        return detected_texts


def extract_voter_id(input):
    regex = r".{0,3}[0-9]{7}"
    match = re.search(regex, input)
    voter_id = match.group(0) if match else ""

    return voter_id


def extract_names(input):
    regex = r"Name\s*[:=+]?\s*(.*)"
    matches = re.findall(regex, input, re.IGNORECASE)
    names = [match.strip() for match in matches] if matches else []

    return names


def extract_lines_with_uppercase_words(input):
    lines_with_uppercase_words = []
    pattern = r"\b[A-Z]+(?:\s+[A-Z]+)*\b"
    for line in input.split("\n"):
        if re.search(pattern, line):
            uppercase_words = re.findall(pattern, line)
            for word in uppercase_words:
                lines_with_uppercase_words.append(word)
    return lines_with_uppercase_words


def extract_gender(input):
    if "Female" in input or "FEMALE" in input:
        return "Female"
    elif "Male" in input or "MALE" in input:
        return "Male"
    else:
        return ""


def extract_date(input):
    regex = r"\b([0-9X]{2}[/\-.][0-9X]{2}[/\-.](?:\d{4}|\d{2}))\b"
    match = re.search(regex, input)
    dob = match.group(0) if match else ""

    return dob


def extract_address(input):
    regex = r"Address\s*:?\s*[A-Za-z0-9:,-.\n\s\/]+[0-9]{6}"
    match = re.search(regex, input)
    address = match.group(0) if match else ""

    if not match:
        regex = r"[A-Za-z0-9:,-.\n\s\/]+[0-9]{6}"
        match = re.search(regex, input)
        address = match.group(0) if match else ""

    return address


def extract_voterid_details_front(image_path):
    image = Image.open(image_path)
    extracted_text = pytesseract.image_to_string(image)

    voter_id = extract_voter_id(extracted_text)

    names = extract_names(extracted_text)
    electors_name = names[0] if len(names) > 0 else ""
    fathers_name = names[1] if len(names) > 1 else ""

    gender = extract_gender(extracted_text)

    dob = extract_date(extracted_text)
    rgb = image.convert("RGB")
    with tempfile.TemporaryDirectory() as tempdir:
        tempfile_path = f"{tempdir}/{str(uuid.uuid4())}.jpg"
        rgb.save(tempfile_path)

        image = cv2.imread(tempfile_path)
        preprocessed = preprocess_for_bold_text(image)
        cv2.imwrite("preprocessed_image.jpg", preprocessed)

        image = Image.open("preprocessed_image.jpg")
        clean_text = pytesseract.image_to_string(image)

        if electors_name == "":
            names = extract_lines_with_uppercase_words(clean_text)
            electors_name = names[-2] if len(names) > 1 else ""
            fathers_name = names[-1] if len(names) > 0 else ""

        if dob == "":
            dob = extract_date(clean_text)

        if voter_id == "":
            voter_id = extract_voter_id(clean_text)

        if gender == "":
            gender = extract_gender(clean_text)

        return {
            "Voter ID": voter_id,
            "Elector's Name": electors_name,
            "Father's Name": fathers_name,
            "Gender": gender,
            "Date of Birth": dob,
        }


def extract_voterid_details_back(image_path):
    image = Image.open(image_path)
    extracted_text = pytesseract.image_to_string(image)

    address = extract_address(extracted_text)
    doi = extract_date(extracted_text)

    return {"Address": address, "Date of Issue": doi}


def voter_id_front(front_path):
    image = Image.open(front_path)
    extracted_text = pytesseract.image_to_string(image)

    if (
        "Date" in extracted_text
        or "Age" in extracted_text
        or "Sex" in extracted_text
        or "Gender" in extracted_text
    ):
        return extract_voterid_details_front(front_path)
    else:
        return extract_voter_details_yolo(front_path)


def voter_id_back(back_path):
    back_details = extract_voterid_details_back(back_path)
    return back_details



import re
import cv2
import pytesseract
from PIL import Image
import tempfile
import uuid


def extract_name(input):
    name_regex = r"\b[A-Z][a-z]+(?:\s+[A-Z][a-z]+)*\b"
    names = re.findall(name_regex, input)
    full_name = ""
    for name in names:
        if "Government" not in name and "India" not in name:
            full_name = name
            break

    return full_name


def extract_fathers_name(input):
    regex = r"(?:S.?O|D.?O)[:\s]*([A-Za-z]+(?: [A-Za-z]+)*)"
    match = re.findall(regex, input)
    fathers_name = ""
    if match:
        fathers_name = match[-1]

    return fathers_name


def extract_aadhaar(input):
    regex = r"\b\d{4}\s?\d{4}\s?\d{4}\b"
    match = re.search(regex, input)
    aadhaar_number = match.group(0) if match else ""

    return aadhaar_number


def extract_dob(input):
    regex = r"\b(\d{2}/\d{2}/\d{4})\b"
    match = re.search(regex, input)
    dob = match.group(0) if match else ""

    return dob


def extract_yob(input):
    regex = r"\b\d{4}\b"
    match = re.search(regex, input)
    yob = match.group(0) if match else ""

    return yob


def extract_gender(input):
    if re.search("Female", input) or re.search("FEMALE", input):
        return "Female"
    if re.search("Male", input) or re.search("MALE", input):
        return "Male"
    return "Other"


def extract_address(input):
    regex = r"Address:\s*((?:.|\n)*?\d{6})"
    match = re.search(regex, input)
    address = match.group(1) if match else ""

    return address


def extract_back_aadhaar_details(image_path):
    image = Image.open(image_path)

    extracted_text = pytesseract.image_to_string(image)

    fathers_name = extract_fathers_name(extracted_text)
    address = extract_address(extracted_text)

    return {
        "Father's Name": fathers_name,
        "Address": address,
    }


def extract_front_aadhaar_details(image_path):
    image = Image.open(image_path)

    extracted_text = pytesseract.image_to_string(image)

    full_name = extract_name(extracted_text)
    dob = extract_dob(extracted_text)
    gender = extract_gender(extracted_text)
    aadhaar_number = extract_aadhaar(extracted_text)

    if dob == "":
        dob = extract_yob(extracted_text)

    return {
        "Full Name": full_name,
        "Date/Year of Birth": dob,
        "Gender": gender,
        "Aadhaar Number": aadhaar_number,
    }


def front_aadhaar(image_path):
    return extract_front_aadhaar_details(image_path)


def back_aadhaar(image_path):
    return extract_back_aadhaar_details(image_path)



# author:    Kunal Kumar Kushwaha
# website:   http://www.essentia.dev

import setuptools
import glob

# Long description
with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fp:
    install_requires = fp.read()

setuptools.setup(
    name="openbharatocr",
    version="0.3.1",
    description="openbharatocr is an opensource python library for ocr Indian government documents",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/essentiasoftserv/openbharatocr",
    author="essentiasoftserv",
    python_requires=">=3.6",
    install_requires=install_requires,
    author_email="kunal@essentia.dev",
    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=False,
    license="Apache",
)



import re
import pytesseract
from PIL import Image
import cv2
import numpy as np


def preprocess_for_bold_text(image):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (1, 1))
    opening = cv2.morphologyEx(gray, cv2.MORPH_OPEN, kernel)
    contrast = cv2.addWeighted(opening, 2, opening, -0.5, 0)

    _, binary = cv2.threshold(contrast, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
    sharpened = cv2.filter2D(
        binary, -1, np.array([[0, -1, 0], [-1, 5, -1], [0, -1, 0]])
    )

    return sharpened


def extract_names(input):
    name_regex = r"Names[\s:]+([A-Za-z\s]+)(?:\n|$)"
    surname_regex = r"Surname[\s:]+([A-Za-z\s]+)(?:\n|$)"

    name_match = re.search(name_regex, input, re.IGNORECASE)
    surname_match = re.search(surname_regex, input, re.IGNORECASE)

    name = name_match.group(1).strip() if name_match else ""
    surname = surname_match.group(1).strip() if surname_match else ""

    return name, surname


def extract_all_dates(input):
    regex = r"\b(\d{2}[/\-.]\d{2}[/\-.](?:\d{4}|\d{2}))\b"
    dates = re.findall(regex, input)
    dates = sorted(dates, key=lambda x: int(re.split(r"[-/]", x)[-1]))

    seen = set()
    unique_dates = []

    for date in dates:
        if date not in seen:
            seen.add(date)
            unique_dates.append(date)

    return unique_dates


def extract_all_places(input):
    dates = re.findall(r"\b(\d{2}[/\-.]\d{2}[/\-.](?:\d{4}|\d{2}))\b", input)
    last_date = dates[-1] if dates else None

    all_places = []

    if last_date:
        places = input.split("\n")
        for place in places:
            if re.match(r'^[A-Z,. -\'"]+$', place) and place.strip():
                all_places.append(place)

    return all_places


def extract_passport_number(input):
    regex = r"[A-Z][0-9]{7}"
    match = re.search(regex, input)
    passport_number = match.group(0) if match else ""

    return passport_number


def extract_details(input):
    lines = input.split("\n")
    clean = []
    for line in lines:
        clean_line = re.sub(r"[^a-zA-Z\s]", "", line)
        clean_line = " ".join(word for word in clean_line.split() if len(word) > 1)
        if (
            re.match(r"^[A-Z\s]{3,}$", clean_line)
            and "INDIA" not in clean_line
            and "REPUBLIC" not in clean_line
            and "PASSPORT" not in clean_line
        ):
            clean.append(clean_line.strip())

    name = clean[1] if len(clean) > 1 else ""
    surname = clean[0] if len(clean) > 0 else ""

    gender = ""

    if len(clean) > 1:
        if "M" in clean[-1]:
            gender = "Male"
        elif "F" in clean[-1]:
            gender = "Female"

    return gender, name, surname


def extract_passport_details(image_path):
    image = Image.open(image_path)
    extracted_text = pytesseract.image_to_string(image)
    image.save("image.jpg", "JPEG")
    image = cv2.imread("image.jpg")
    preprocessed = preprocess_for_bold_text(image)
    cv2.imwrite("preprocessed_image.jpg", preprocessed)

    image = Image.open("preprocessed_image.jpg")
    clean_text = pytesseract.image_to_string(image)

    dates = extract_all_dates(extracted_text)
    dob = dates[0] if len(dates) > 0 else ""
    doi = dates[1] if len(dates) > 1 else ""
    expiry_date = dates[2] if len(dates) > 2 else ""

    passport_number = extract_passport_number(extracted_text)

    places = extract_all_places(extracted_text)
    pob = places[-2] if len(places) > 1 else ""
    poi = places[-1] if len(places) > 0 else ""

    gender, name, surname = extract_details(clean_text)

    return {
        "Name": name,
        "Surname": surname,
        "Passport Number": passport_number,
        "Gender": gender,
        "Place of Birth": pob,
        "Date of Birth": dob,
        "Place of Issue": poi,
        "Date of Issue": doi,
        "Expiry Date": expiry_date,
    }


def passport(image_path):
    return extract_passport_details(image_path)
